@@ -47,7 +47,7 @@
 #endif
 
 #define KE_TIME_WAIT 			5000
-#define COUNT_OF_ITEM_IN_LAYER_KEYSET 	16
+#define COUNT_OF_ITEM_IN_LAYER_KEYSET 	17
 
 #define MODIFIER(K,C) (K ? C : "")
 #define LINE(X)       (X != 0 ? "-" : "")
@@ -74,24 +74,6 @@ typedef struct
     SREvent *evnt;
 } IdleStruct;    
     
-#define KE_INTERESTED_MODIFIERS_NUMBER 6
-struct
-{
-    gint  	modifier;
-    KeySym 	keysym;
-    KeyCode     keyCode;
-    u_int32_t   byteIndex;
-    u_int32_t   bitIndex;
-} KeModifierMap [KE_INTERESTED_MODIFIERS_NUMBER] =
-{
-    {SPI_KEYMASK_SHIFT, 	XK_Shift_L, 	0, 0, 0},
-    {SPI_KEYMASK_SHIFT, 	XK_Shift_R, 	0, 0, 0},
-    {SPI_KEYMASK_CONTROL, 	XK_Control_L, 	0, 0, 0},
-    {SPI_KEYMASK_CONTROL, 	XK_Control_R, 	0, 0, 0},
-    {SPI_KEYMASK_ALT, 		XK_Alt_L, 	0, 0, 0},
-    {SPI_KEYMASK_ALT, 		XK_Alt_R, 	0, 0, 0}
-};
-
 /**
  *
  * Key string translation table.
@@ -401,6 +383,7 @@ ke_return_keystring (gint keyID)
         case XK_Page_Down:rv = g_strdup("Page_Down");break;
 	case XK_BackSpace:rv = g_strdup ("BackSpace");break;
 	case XK_Delete:rv = g_strdup ("Delete");break;
+	case XK_Escape:rv = g_strdup ("Escape");break;
         default:
     	    rv = NULL;
     }
@@ -570,6 +553,7 @@ ke_return_key (glong keyID)
 	case XK_KP_8:		layer_code = 8; break;
 	case XK_KP_9:		layer_code = 9; break;
 	case XK_KP_Decimal:	layer_code = 10; break;
+	case XK_KP_Separator:	layer_code = 10; break;
 	case XK_KP_Enter:	layer_code = 11; break;
 	case XK_KP_Add:		layer_code = 12; break;
 	case XK_KP_Subtract:	layer_code = 13; break;
@@ -1024,52 +1008,6 @@ ke_keysym_to_string (glong keysym, gint modifier)
     return keystring;
 }
 
-/**
- * ke_return_active_modifiers
- *
- *
- * Return state of Shifts, Alts, Controls and Num Lock.
- *
-**/
-gint
-ke_return_active_modifiers (void)
-{
-    gchar       keyVector[32];
-    Display    *display;
-    gint        i;
-    gint 	rv = 0;
-
-    if ((display = XOpenDisplay(NULL) ) == NULL ) 
-    {
-        sru_warning ("Unable to open display.");
-        return 0;
-    }
-    
-    /* Return the keyvector */
-    XQueryKeymap (display, keyVector);
-        
-    /*Return state of Shifts, Alts, Controls and for the Num Lock */
-    for (i = 0 ; i < G_N_ELEMENTS (KeModifierMap) ; i++)
-    {
-	KeModifierMap[i].keyCode = XKeysymToKeycode (display, KeModifierMap[i].keysym);
-
-	if (KeModifierMap[i].keyCode == 0) 
-	{
-    	    sru_warning ("Invalid KeyCode.");
-    	    return 0;
-	}
-                                                                                
-	KeModifierMap[i].byteIndex = (KeModifierMap[i].keyCode/8);
-	KeModifierMap[i].bitIndex  = KeModifierMap[i].keyCode & 7;
-	
-	if (1 & (keyVector[KeModifierMap[i].byteIndex] >> KeModifierMap[i].bitIndex)) 
-	    rv = rv | KeModifierMap[i].modifier;
-
-    }
-    
-    return rv;
-}
-
 /**
 *
 * ke_report_user_key  
@@ -1087,7 +1025,6 @@ ke_report_user_key_event (const AccessibleKeystroke *key, gpointer user_data)
     static gboolean user_busy = FALSE;    
     static GQueue  *user_queue = NULL;
     KEKeyEvent     *ev;
-
     sru_debug ("(User)Received key event:\n\tsym %ld (%c)\n\tmods %x\n\tcode %d\n\ttime %ld\n\tkeystring \"%s\"\n\ttype %d (press = %d, release = %d)",
 	      (glong) key->keyID, (gchar)key->keyID,
 	      (guint) key->modifiers,
@@ -1139,7 +1076,10 @@ ke_report_user_key_event (const AccessibleKeystroke *key, gpointer user_data)
 	/* pop from queue the event */
 	key1 = (KEKeyEvent *) g_queue_pop_tail (user_queue);
 	/* get the status of modifiers */
-	modif  = ke_return_active_modifiers ();
+	if (key->modifiers & (~(SPI_KEYMASK_ALT | SPI_KEYMASK_CONTROL | SPI_KEYMASK_SHIFT)))
+	    sru_warning ("Callback invoked with an unexpected modifier.");
+	modif  = key1->modifiers & (SPI_KEYMASK_ALT | SPI_KEYMASK_CONTROL | SPI_KEYMASK_SHIFT);
+	
 	/* get the keystring */
 	keystr = ke_keysym_to_string (key1->keyID, modif);
         /* create a command */
@@ -1488,6 +1428,8 @@ ke_get_keypad_keycodes (void)
     keycodes [13] = XKeysymToKeycode (display, XK_KP_Subtract);
     keycodes [14] = XKeysymToKeycode (display, XK_KP_Multiply);
     keycodes [15] = XKeysymToKeycode (display, XK_KP_Divide);
+    /* Fix for bug #148774 - some kbds report XK_KP_Separator in numpad */
+    keycodes [16] = XKeysymToKeycode (display, XK_KP_Separator);
     	
     return keycodes;
 }
@@ -1519,7 +1461,8 @@ long keyecho_keysym[] = {
     XK_Page_Up,
     XK_Page_Down,
     XK_BackSpace,
-    XK_Delete
+    XK_Delete,
+    XK_Escape
 };
 
 gchar*
